"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../util");
const lodash_get_1 = __importDefault(require("lodash.get"));
const lodash_has_1 = __importDefault(require("lodash.has"));
class Validator {
    constructor(errors = {}) {
        this.processing = false;
        this.successful = false;
        this.errors = errors;
    }
    add(attribute, message) {
        if (this.missed(attribute)) {
            this.errors[attribute] = [];
        }
        if (!this.errors[attribute].includes(message)) {
            this.errors[attribute].push(message);
        }
    }
    has(field) {
        if (Array.isArray(field)) {
            return (0, util_1.is)(Object.keys(this.errors), field);
        }
        let hasError = (0, lodash_has_1.default)(this.errors, field);
        if (!hasError) {
            const errors = Object.keys(this.errors).filter((e) => e.startsWith(`${field}.`) || e.startsWith(`${field}[`));
            hasError = errors.length > 0;
        }
        return hasError;
    }
    first(field) {
        if (Array.isArray(field)) {
            for (let i = 0; i < field.length; i++) {
                if (!(0, lodash_has_1.default)(this.errors, field[i])) {
                    continue;
                }
                return this.first(field[i]);
            }
        }
        const value = this.get(field);
        if (Array.isArray(value))
            return value[0];
        return value; // return it if object like
    }
    firstBy(obj, field) {
        let value;
        if (!field) {
            value = obj[Object.keys(obj)[0]];
        }
        else {
            value = obj[field];
        }
        if (Array.isArray(value))
            value = value[0];
        return value;
    }
    missed(field) {
        return !this.has(field);
    }
    nullState(field) {
        return this.has(field) ? this.missed(field) : null;
    }
    any(fields = [], returnObject) {
        if (returnObject) {
            const errors = {};
            if (!fields.length) {
                return {};
            }
            fields.forEach((key) => (errors[key] = this.get(key)));
            return errors;
        }
        if (!fields.length) {
            return Object.keys(this.errors).length > 0;
        }
        const errors = {};
        fields.forEach((key) => (errors[key] = this.get(key)));
        return Object.keys(errors).length > 0;
    }
    get(field) {
        return (0, lodash_get_1.default)(this.errors, field) || [];
    }
    all() {
        return this.errors;
    }
    count() {
        return Object.keys(this.errors).length;
    }
    fill(errors) {
        this.errors = errors;
    }
    flush() {
        this.errors = {};
    }
    clear(attribute) {
        if (!attribute) {
            return this.flush();
        }
        const errors = Object.assign({}, this.errors);
        if (attribute instanceof Array) {
            attribute.map((field) => {
                Object.keys(errors)
                    .filter((e) => e === field ||
                    e.startsWith(`${field}.`) ||
                    e.startsWith(`${field}[`))
                    .forEach((e) => delete errors[e]);
            });
        }
        else {
            Object.keys(errors)
                .filter((e) => e === attribute ||
                e.startsWith(`${attribute}.`) ||
                e.startsWith(`${attribute}[`))
                .forEach((e) => delete errors[e]);
        }
        this.fill(errors);
    }
    isValid() {
        return !this.any();
    }
    onKeydown(event, prefix) {
        const { name } = event.target;
        if (!name)
            return;
        const nameWithPrefix = prefix ? `${prefix}.${name}` : null;
        this.clear([name, nameWithPrefix]);
    }
}
exports.default = new Validator();
